@@ -58,25 +58,34 @@ def raw_template_delete(context, template_id):
     return IMPL.raw_template_delete(context, template_id)
 
 
+def raw_template_files_create(context, values):
+    return IMPL.raw_template_files_create(context, values)
+
+
+def raw_template_files_get(context, tmpl_files_id):
+    return IMPL.raw_template_files_get(context, tmpl_files_id)
+
+
 def resource_data_get_all(context, resource_id, data=None):
     return IMPL.resource_data_get_all(context, resource_id, data)
 
 
-def resource_data_get(resource, key):
-    return IMPL.resource_data_get(resource, key)
+def resource_data_get(context, resource_id, key):
+    return IMPL.resource_data_get(context, resource_id, key)
 
 
-def resource_data_set(resource, key, value, redact=False):
-    return IMPL.resource_data_set(resource, key, value, redact=redact)
+def resource_data_set(context, resource_id, key, value, redact=False):
+    return IMPL.resource_data_set(context, resource_id, key, value,
+                                  redact=redact)
 
 
 def resource_data_get_by_key(context, resource_id, key):
     return IMPL.resource_data_get_by_key(context, resource_id, key)
 
 
-def resource_data_delete(resource, key):
+def resource_data_delete(context, resource_id, key):
     """Remove a resource_data element associated to a resource."""
-    return IMPL.resource_data_delete(resource, key)
+    return IMPL.resource_data_delete(context, resource_id, key)
 
 
 def stack_tags_set(context, stack_id, tags):
@@ -91,8 +100,8 @@ def stack_tags_get(context, stack_id):
     return IMPL.stack_tags_get(context, stack_id)
 
 
-def resource_get(context, resource_id):
-    return IMPL.resource_get(context, resource_id)
+def resource_get(context, resource_id, refresh=False):
+    return IMPL.resource_get(context, resource_id, refresh=refresh)
 
 
 def resource_get_all(context):
@@ -105,16 +114,40 @@ def resource_update(context, resource_id, values, atomic_key,
                                 expected_engine_id)
 
 
+def resource_update_and_save(context, resource_id, values):
+    return IMPL.resource_update_and_save(context, resource_id, values)
+
+
 def resource_create(context, values):
     return IMPL.resource_create(context, values)
 
 
+def resource_delete(context, resource_id):
+    return IMPL.resource_delete(context, resource_id)
+
+
 def resource_exchange_stacks(context, resource_id1, resource_id2):
     return IMPL.resource_exchange_stacks(context, resource_id1, resource_id2)
 
 
-def resource_get_all_by_stack(context, stack_id, key_id=False, filters=None):
-    return IMPL.resource_get_all_by_stack(context, stack_id, key_id, filters)
+def resource_get_all_by_stack(context, stack_id, filters=None):
+    return IMPL.resource_get_all_by_stack(context, stack_id, filters)
+
+
+def resource_get_all_active_by_stack(context, stack_id):
+    return IMPL.resource_get_all_active_by_stack(context, stack_id)
+
+
+def resource_get_all_by_root_stack(context, stack_id, filters=None):
+    return IMPL.resource_get_all_by_root_stack(context, stack_id, filters)
+
+
+def engine_get_all_locked_by_stack(context, stack_id):
+    return IMPL.engine_get_all_locked_by_stack(context, stack_id)
+
+
+def resource_purge_deleted(context, stack_id):
+    return IMPL.resource_purge_deleted(context, stack_id)
 
 
 def resource_get_by_name_and_stack(context, resource_name, stack_id):
@@ -127,11 +160,8 @@ def resource_get_by_physical_resource_id(context, physical_resource_id):
                                                      physical_resource_id)
 
 
-def stack_get(context, stack_id, show_deleted=False, tenant_safe=True,
-              eager_load=False):
-    return IMPL.stack_get(context, stack_id, show_deleted=show_deleted,
-                          tenant_safe=tenant_safe,
-                          eager_load=eager_load)
+def stack_get(context, stack_id, show_deleted=False):
+    return IMPL.stack_get(context, stack_id, show_deleted=show_deleted)
 
 
 def stack_get_status(context, stack_id):
@@ -148,12 +178,12 @@ def stack_get_by_name(context, stack_name):
 
 
 def stack_get_all(context, limit=None, sort_keys=None, marker=None,
-                  sort_dir=None, filters=None, tenant_safe=True,
+                  sort_dir=None, filters=None,
                   show_deleted=False, show_nested=False, show_hidden=False,
                   tags=None, tags_any=None, not_tags=None,
                   not_tags_any=None):
     return IMPL.stack_get_all(context, limit, sort_keys,
-                              marker, sort_dir, filters, tenant_safe,
+                              marker, sort_dir, filters,
                               show_deleted, show_nested, show_hidden,
                               tags, tags_any, not_tags, not_tags_any)
 
@@ -162,12 +192,15 @@ def stack_get_all_by_owner_id(context, owner_id):
     return IMPL.stack_get_all_by_owner_id(context, owner_id)
 
 
-def stack_count_all(context, filters=None, tenant_safe=True,
+def stack_get_all_by_root_owner_id(context, owner_id):
+    return IMPL.stack_get_all_by_root_owner_id(context, owner_id)
+
+
+def stack_count_all(context, filters=None,
                     show_deleted=False, show_nested=False, show_hidden=False,
                     tags=None, tags_any=None, not_tags=None,
                     not_tags_any=None):
     return IMPL.stack_count_all(context, filters=filters,
-                                tenant_safe=tenant_safe,
                                 show_deleted=show_deleted,
                                 show_nested=show_nested,
                                 show_hidden=show_hidden,
@@ -189,20 +222,21 @@ def stack_delete(context, stack_id):
     return IMPL.stack_delete(context, stack_id)
 
 
-def stack_lock_create(stack_id, engine_id):
-    return IMPL.stack_lock_create(stack_id, engine_id)
+def stack_lock_create(context, stack_id, engine_id):
+    return IMPL.stack_lock_create(context, stack_id, engine_id)
 
 
-def stack_lock_get_engine_id(stack_id):
-    return IMPL.stack_lock_get_engine_id(stack_id)
+def stack_lock_get_engine_id(context, stack_id):
+    return IMPL.stack_lock_get_engine_id(context, stack_id)
 
 
-def stack_lock_steal(stack_id, old_engine_id, new_engine_id):
-    return IMPL.stack_lock_steal(stack_id, old_engine_id, new_engine_id)
+def stack_lock_steal(context, stack_id, old_engine_id, new_engine_id):
+    return IMPL.stack_lock_steal(context, stack_id, old_engine_id,
+                                 new_engine_id)
 
 
-def stack_lock_release(stack_id, engine_id):
-    return IMPL.stack_lock_release(stack_id, engine_id)
+def stack_lock_release(context, stack_id, engine_id):
+    return IMPL.stack_lock_release(context, stack_id, engine_id)
 
 
 def persist_state_and_release_lock(context, stack_id, engine_id, values):
@@ -226,8 +260,8 @@ def user_creds_delete(context, user_creds_id):
     return IMPL.user_creds_delete(context, user_creds_id)
 
 
-def user_creds_get(context_id):
-    return IMPL.user_creds_get(context_id)
+def user_creds_get(context, user_creds_id):
+    return IMPL.user_creds_get(context, user_creds_id)
 
 
 def event_get(context, event_id):
@@ -314,12 +348,10 @@ def software_config_get(context, config_id):
     return IMPL.software_config_get(context, config_id)
 
 
-def software_config_get_all(context, limit=None, marker=None,
-                            tenant_safe=True):
+def software_config_get_all(context, limit=None, marker=None):
     return IMPL.software_config_get_all(context,
                                         limit=limit,
-                                        marker=marker,
-                                        tenant_safe=tenant_safe)
+                                        marker=marker)
 
 
 def software_config_delete(context, config_id):
@@ -425,3 +457,7 @@ def db_sync(engine, version=None):
 def db_version(engine):
     """Display the current database version."""
     return IMPL.db_version(engine)
+
+
+def reset_stack_status(context, stack_id):
+    return IMPL.reset_stack_status(context, stack_id)
