@@ -40,7 +40,7 @@ from heat.engine import constraints
 LOG = logging.getLogger(__name__)
 
 
-NOVACLIENT_VERSION = "2"
+NOVA_API_VERSION = "2.1"
 CLIENT_NAME = 'nova'
 
 
@@ -63,29 +63,19 @@ class NovaClientPlugin(client_plugin.ClientPlugin):
 
     def _create(self):
         endpoint_type = self._get_client_option(CLIENT_NAME, 'endpoint_type')
-        management_url = self.url_for(service_type=self.COMPUTE,
-                                      endpoint_type=endpoint_type)
-        extensions = nc.discover_extensions(NOVACLIENT_VERSION)
+        extensions = nc.discover_extensions(NOVA_API_VERSION)
 
         args = {
-            'project_id': self.context.tenant_id,
-            'auth_url': self.context.auth_url,
-            'auth_token': self.auth_token,
-            'service_type': self.COMPUTE,
-            'username': None,
-            'api_key': None,
+            'session': self.context.keystone_session,
             'extensions': extensions,
-            'endpoint_type': endpoint_type,
+            'interface': endpoint_type,
+            'service_type': self.COMPUTE,
+            'region_name': self._get_region_name(),
             'http_log_debug': self._get_client_option(CLIENT_NAME,
-                                                      'http_log_debug'),
-            'cacert': self._get_client_option(CLIENT_NAME, 'ca_file'),
-            'insecure': self._get_client_option(CLIENT_NAME, 'insecure')
+                                                      'http_log_debug')
         }
 
-        client = nc.Client(NOVACLIENT_VERSION, **args)
-
-        client.client.set_management_url(management_url)
-
+        client = nc.Client(NOVA_API_VERSION, **args)
         return client
 
     def is_not_found(self, ex):
@@ -311,15 +301,7 @@ class NovaClientPlugin(client_plugin.ClientPlugin):
                 subtype = os.path.splitext(filename)[0]
             if content is None:
                 content = ''
-
-            try:
-                content.encode('us-ascii')
-                charset = 'us-ascii'
-            except UnicodeEncodeError:
-                charset = 'utf-8'
-            msg = (text.MIMEText(content, _subtype=subtype, _charset=charset)
-                   if subtype else text.MIMEText(content, _charset=charset))
-
+            msg = text.MIMEText(content, _subtype=subtype)
             msg.add_header('Content-Disposition', 'attachment',
                            filename=filename)
             return msg
@@ -504,13 +486,12 @@ echo -e '%s\tALL=(ALL)\tNOPASSWD: ALL' >> /etc/sudoers
                 result=msg, resource_status=status)
 
     def rebuild(self, server_id, image_id, password=None,
-                preserve_ephemeral=False, meta=None, files=None):
+                preserve_ephemeral=False):
         """Rebuild the server and call check_rebuild to verify."""
         server = self.fetch_server(server_id)
         if server:
             server.rebuild(image_id, password=password,
-                           preserve_ephemeral=preserve_ephemeral,
-                           meta=meta, files=files)
+                           preserve_ephemeral=preserve_ephemeral)
             return True
         else:
             return False
@@ -544,8 +525,7 @@ echo -e '%s\tALL=(ALL)\tNOPASSWD: ALL' >> /etc/sudoers
         """Delete/Add the metadata in nova as needed."""
         metadata = self.meta_serialize(metadata)
         current_md = server.metadata
-        to_del = sorted([key for key in six.iterkeys(current_md)
-                         if key not in metadata])
+        to_del = sorted(set(current_md) - set(metadata))
         client = self.client()
         if len(to_del) > 0:
             client.servers.delete_meta(server, to_del)
