@@ -20,7 +20,6 @@ from heat.common import identifier
 from heat.engine.clients.os import keystone
 from heat.engine import dependencies
 from heat.engine import resource as res
-from heat.engine.resources.aws.ec2 import instance as ins
 from heat.engine import service
 from heat.engine import stack
 from heat.engine import stack_lock
@@ -264,20 +263,6 @@ class StackResourcesServiceTest(common.HeatTestCase):
         self.stack.iter_resources.assert_called_once_with(max_depth,
                                                           filters=None)
 
-    @mock.patch.object(stack.Stack, 'load')
-    @tools.stack_context('service_resources_list_test_stack')
-    def test_stack_resources_filter_id(self, mock_load):
-        mock_load.return_value = self.stack
-        resources = six.itervalues(self.stack)
-        self.stack.iter_resources = mock.Mock(return_value=resources)
-        filters = {'physical_resource_id': '123'}
-        self.eng.list_stack_resources(self.ctx,
-                                      self.stack.identifier(),
-                                      filters=filters)
-        expected_filters = {'nova_instance': '123'}
-        self.stack.iter_resources.assert_called_once_with(
-            0, filters=expected_filters)
-
     @mock.patch.object(stack.Stack, 'load')
     @tools.stack_context('service_resources_list_test_stack')
     def test_stack_resources_filter_type(self, mock_load):
@@ -313,11 +298,7 @@ class StackResourcesServiceTest(common.HeatTestCase):
         mock_load.return_value = stk
         tools.clean_up_stack(stk)
         resources = self.eng.list_stack_resources(self.ctx, stack_id)
-        self.assertEqual(1, len(resources))
-
-        res = resources[0]
-        self.assertEqual('DELETE', res['resource_action'])
-        self.assertEqual('COMPLETE', res['resource_status'])
+        self.assertEqual(0, len(resources))
 
     @mock.patch.object(service.EngineService, '_get_stack')
     def test_stack_resources_list_nonexist_stack(self, mock_get):
@@ -351,7 +332,6 @@ class StackResourcesServiceTest(common.HeatTestCase):
         stack_name = 'signal_reception_async'
         self.stack = self._stack_create(stack_name)
         test_data = {'food': 'yum'}
-
         self.eng.resource_signal(self.ctx,
                                  dict(self.stack.identifier()),
                                  'WebServerScaleDownPolicy',
@@ -375,6 +355,20 @@ class StackResourcesServiceTest(common.HeatTestCase):
                                  sync_call=True)
         mock_signal.assert_called_once_with(mock.ANY, False)
 
+    def test_signal_reception_get_resource_none(self):
+        stack_name = 'signal_reception_no_resource'
+        self.stack = self._stack_create(stack_name)
+        test_data = {'food': 'yum'}
+
+        self.patchobject(stack.Stack, 'resource_get',
+                         return_value=None)
+        ex = self.assertRaises(dispatcher.ExpectedException,
+                               self.eng.resource_signal, self.ctx,
+                               dict(self.stack.identifier()),
+                               'WebServerScaleDownPolicy',
+                               test_data)
+        self.assertEqual(exception.ResourceNotFound, ex.exc_info[0])
+
     def test_signal_reception_no_resource(self):
         stack_name = 'signal_reception_no_resource'
         self.stack = self._stack_create(stack_name)
@@ -542,77 +536,82 @@ class StackResourcesServiceTest(common.HeatTestCase):
         self.assertIsInstance(stack_dependencies, dependencies.Dependencies)
         self.assertEqual(2, len(stack_dependencies.graph()))
 
-    def _test_mark_healthy_asserts(self, action='CHECK', status='FAILED',
-                                   reason='state changed', meta=None):
-        rs = self.eng.describe_stack_resource(
-            self.ctx, self.stack.identifier(),
-            'WebServer', with_attr=None)
-        self.assertIn('resource_action', rs)
-        self.assertIn('resource_status', rs)
-        self.assertIn('resource_status_reason', rs)
-
-        self.assertEqual(action, rs['resource_action'])
-        self.assertEqual(status, rs['resource_status'])
-        self.assertEqual(reason, rs['resource_status_reason'])
-        if meta is not None:
-            self.assertIn('metadata', rs)
-            self.assertEqual(meta, rs['metadata'])
-
     @tools.stack_context('service_mark_healthy_create_complete_test_stk')
     def test_mark_healthy_in_create_complete(self):
         self.eng.resource_mark_unhealthy(self.ctx, self.stack.identifier(),
                                          'WebServer', False,
                                          resource_status_reason='noop')
 
-        self._test_mark_healthy_asserts(action='CREATE',
-                                        status='COMPLETE')
+        r = self.eng.describe_stack_resource(self.ctx, self.stack.identifier(),
+                                             'WebServer', with_attr=None)
+        self.assertIn('resource_action', r)
+        self.assertIn('resource_status', r)
+        self.assertIn('resource_status_reason', r)
+
+        self.assertEqual(r['resource_action'], 'CREATE')
+        self.assertEqual(r['resource_status'], 'COMPLETE')
+        self.assertEqual(r['resource_status_reason'], 'state changed')
 
     @tools.stack_context('service_mark_unhealthy_create_complete_test_stk')
     def test_mark_unhealthy_in_create_complete(self):
-
-        reason = 'Some Reason'
         self.eng.resource_mark_unhealthy(self.ctx, self.stack.identifier(),
                                          'WebServer', True,
-                                         resource_status_reason=reason)
+                                         resource_status_reason='Some Reason')
 
-        self._test_mark_healthy_asserts(reason=reason)
+        r = self.eng.describe_stack_resource(self.ctx, self.stack.identifier(),
+                                             'WebServer', with_attr=None)
+
+        self.assertEqual(r['resource_action'], 'CHECK')
+        self.assertEqual(r['resource_status'], 'FAILED')
+        self.assertEqual(r['resource_status_reason'], 'Some Reason')
 
     @tools.stack_context('service_mark_healthy_check_failed_test_stk')
     def test_mark_healthy_check_failed(self):
-        reason = 'Some Reason'
         self.eng.resource_mark_unhealthy(self.ctx, self.stack.identifier(),
                                          'WebServer', True,
-                                         resource_status_reason=reason)
-        self._test_mark_healthy_asserts(reason=reason)
+                                         resource_status_reason='Some Reason')
 
-        meta = {'for_test': True}
+        r = self.eng.describe_stack_resource(self.ctx, self.stack.identifier(),
+                                             'WebServer', with_attr=None)
 
-        def override_metadata_reset(rsrc):
-            rsrc.metadata_set(meta)
+        self.assertEqual(r['resource_action'], 'CHECK')
+        self.assertEqual(r['resource_status'], 'FAILED')
+        self.assertEqual(r['resource_status_reason'], 'Some Reason')
 
-        ins.Instance.handle_metadata_reset = override_metadata_reset
-
-        reason = 'Good Reason'
         self.eng.resource_mark_unhealthy(self.ctx, self.stack.identifier(),
                                          'WebServer', False,
-                                         resource_status_reason=reason)
-        self._test_mark_healthy_asserts(status='COMPLETE',
-                                        reason=reason,
-                                        meta=meta)
+                                         resource_status_reason='Good Reason')
+
+        r = self.eng.describe_stack_resource(self.ctx, self.stack.identifier(),
+                                             'WebServer', with_attr=None)
+
+        self.assertEqual(r['resource_action'], 'CHECK')
+        self.assertEqual(r['resource_status'], 'COMPLETE')
+        self.assertEqual(r['resource_status_reason'], 'Good Reason')
 
     @tools.stack_context('service_mark_unhealthy_check_failed_test_stack')
     def test_mark_unhealthy_check_failed(self):
-        reason = 'Some Reason'
         self.eng.resource_mark_unhealthy(self.ctx, self.stack.identifier(),
                                          'WebServer', True,
-                                         resource_status_reason=reason)
-        self._test_mark_healthy_asserts(reason=reason)
+                                         resource_status_reason='Some Reason')
+
+        r = self.eng.describe_stack_resource(self.ctx, self.stack.identifier(),
+                                             'WebServer', with_attr=None)
+
+        self.assertEqual(r['resource_action'], 'CHECK')
+        self.assertEqual(r['resource_status'], 'FAILED')
+        self.assertEqual(r['resource_status_reason'], 'Some Reason')
 
-        new_reason = 'New Reason'
         self.eng.resource_mark_unhealthy(self.ctx, self.stack.identifier(),
                                          'WebServer', True,
-                                         resource_status_reason=new_reason)
-        self._test_mark_healthy_asserts(reason=new_reason)
+                                         resource_status_reason='New Reason')
+
+        r = self.eng.describe_stack_resource(self.ctx, self.stack.identifier(),
+                                             'WebServer', with_attr=None)
+
+        self.assertEqual(r['resource_action'], 'CHECK')
+        self.assertEqual(r['resource_status'], 'FAILED')
+        self.assertEqual(r['resource_status_reason'], 'New Reason')
 
     @tools.stack_context('service_mark_unhealthy_invalid_value_test_stk')
     def test_mark_unhealthy_invalid_value(self):
@@ -628,16 +627,28 @@ class StackResourcesServiceTest(common.HeatTestCase):
     def test_mark_unhealthy_none_reason(self):
         self.eng.resource_mark_unhealthy(self.ctx, self.stack.identifier(),
                                          'WebServer', True)
-        default_reason = 'state changed by resource_mark_unhealthy api'
-        self._test_mark_healthy_asserts(reason=default_reason)
+
+        r = self.eng.describe_stack_resource(self.ctx, self.stack.identifier(),
+                                             'WebServer', with_attr=None)
+
+        self.assertEqual(r['resource_action'], 'CHECK')
+        self.assertEqual(r['resource_status'], 'FAILED')
+        self.assertEqual(r['resource_status_reason'],
+                         'state changed by resource_mark_unhealthy api')
 
     @tools.stack_context('service_mark_unhealthy_empty_reason_test_stk')
     def test_mark_unhealthy_empty_reason(self):
         self.eng.resource_mark_unhealthy(self.ctx, self.stack.identifier(),
                                          'WebServer', True,
                                          resource_status_reason="")
-        default_reason = 'state changed by resource_mark_unhealthy api'
-        self._test_mark_healthy_asserts(reason=default_reason)
+
+        r = self.eng.describe_stack_resource(self.ctx, self.stack.identifier(),
+                                             'WebServer', with_attr=None)
+
+        self.assertEqual(r['resource_action'], 'CHECK')
+        self.assertEqual(r['resource_status'], 'FAILED')
+        self.assertEqual(r['resource_status_reason'],
+                         'state changed by resource_mark_unhealthy api')
 
     @tools.stack_context('service_mark_unhealthy_lock_no_converge_test_stk')
     def test_mark_unhealthy_lock_no_convergence(self):
