@@ -36,7 +36,7 @@ class GenericResource(resource.Resource):
 
     @classmethod
     def is_service_available(cls, context):
-        return True
+        return (True, None)
 
     def handle_create(self):
         LOG.warning(_LW('Creating generic resource (Type "%s")'),
@@ -62,6 +62,59 @@ class GenericResource(resource.Resource):
                     self.type())
 
 
+class CancellableResource(GenericResource):
+    def check_create_complete(self, cookie):
+        return True
+
+    def handle_create_cancel(self, cookie):
+        LOG.warning(_LW('Cancelling create generic resource (Type "%s")'),
+                    self.type())
+
+    def check_update_complete(self, cookie):
+        return True
+
+    def handle_update_cancel(self, cookie):
+        LOG.warning(_LW('Cancelling update generic resource (Type "%s")'),
+                    self.type())
+
+
+class MultiStepResource(GenericResource):
+    properties_schema = {
+        'create_steps': properties.Schema(properties.Schema.INTEGER,
+                                          default=2),
+        'update_steps': properties.Schema(properties.Schema.INTEGER,
+                                          default=2, update_allowed=True),
+        'delete_steps': properties.Schema(properties.Schema.INTEGER,
+                                          default=2, update_allowed=True),
+    }
+
+    def handle_create(self):
+        super(MultiStepResource, self).handle_create()
+        return [None] * self.properties['create_steps']
+
+    def check_create_complete(self, cookie):
+        cookie.pop()
+        return not cookie
+
+    def handle_update(self, json_snippet, tmpl_diff, prop_diff):
+        super(MultiStepResource, self).handle_update(json_snippet,
+                                                     tmpl_diff,
+                                                     prop_diff)
+        return [None] * self.properties['update_steps']
+
+    def check_update_complete(self, cookie):
+        cookie.pop()
+        return not cookie
+
+    def handle_delete(self):
+        super(MultiStepResource, self).handle_delete()
+        return [None] * self.properties['delete_steps']
+
+    def check_delete_complete(self, cookie):
+        cookie.pop()
+        return not cookie
+
+
 class ResWithShowAttr(GenericResource):
     def _show_resource(self):
         return {'foo': self.name,
@@ -199,6 +252,7 @@ class SignalResource(signal_responder.SignalResponder):
                          'signal': attributes.Schema('Get a signal')}
 
     def handle_create(self):
+        self.password = 'password'
         super(SignalResource, self).handle_create()
         self.resource_id_set(self._get_user_id())
 
