@@ -18,11 +18,15 @@ import uuid
 import mock
 import six
 
+from oslo_serialization import jsonutils
+
 from heat.common import exception as exc
 from heat.common.i18n import _
+from heat.common import template_format
 from heat.engine.clients.os import nova
 from heat.engine.clients.os import swift
 from heat.engine.clients.os import zaqar
+from heat.engine import resource
 from heat.engine.resources.openstack.heat import software_deployment as sd
 from heat.engine import rsrc_defn
 from heat.engine import stack as parser
@@ -132,6 +136,21 @@ class SoftwareDeploymentTest(common.HeatTestCase):
         }
     }
 
+    template_update_only = {
+        'HeatTemplateFormatVersion': '2012-12-12',
+        'Resources': {
+            'deployment_mysql': {
+                'Type': 'OS::Heat::SoftwareDeployment',
+                'Properties': {
+                    'server': '9f1f0e00-05d2-4ca5-8602-95021f19c9d0',
+                    'config': '48e8ade1-9196-42d5-89a2-f709fde42632',
+                    'input_values': {'foo': 'bar'},
+                    'actions': ['UPDATE'],
+                }
+            }
+        }
+    }
+
     template_no_config = {
         'HeatTemplateFormatVersion': '2012-12-12',
         'Resources': {
@@ -189,9 +208,10 @@ class SoftwareDeploymentTest(common.HeatTestCase):
         props = template['Resources']['server']['Properties']
         props['user_data_format'] = 'SOFTWARE_CONFIG'
         self._create_stack(self.template_with_server)
-        sd = self.deployment
-        self.assertEqual('CFN_SIGNAL', sd.properties.get('signal_transport'))
-        sd.validate()
+        mock_sd = self.deployment
+        self.assertEqual('CFN_SIGNAL',
+                         mock_sd.properties.get('signal_transport'))
+        mock_sd.validate()
 
     def test_validate_without_server(self):
         stack = utils.parse_stack(self.template_no_server)
@@ -207,8 +227,8 @@ class SoftwareDeploymentTest(common.HeatTestCase):
         props = template['Resources']['server']['Properties']
         props['user_data_format'] = 'RAW'
         self._create_stack(template)
-        sd = self.deployment
-        err = self.assertRaises(exc.StackValidationFailed, sd.validate)
+        mock_sd = self.deployment
+        err = self.assertRaises(exc.StackValidationFailed, mock_sd.validate)
         self.assertEqual("Resource server's property "
                          "user_data_format should be set to "
                          "SOFTWARE_CONFIG since there are "
@@ -216,7 +236,6 @@ class SoftwareDeploymentTest(common.HeatTestCase):
 
     def mock_software_config(self):
         config = {
-            'id': '48e8ade1-9196-42d5-89a2-f709fde42632',
             'group': 'Test::Group',
             'name': 'myconfig',
             'config': 'the config',
@@ -229,10 +248,37 @@ class SoftwareDeploymentTest(common.HeatTestCase):
                 'name': 'bar',
                 'type': 'String',
                 'default': 'baz',
+            }, {
+                'name': 'trigger_replace',
+                'type': 'String',
+                'default': 'default_value',
+                'replace_on_change': True,
             }],
             'outputs': [],
         }
-        self.rpc_client.show_software_config.return_value = config
+
+        derived_config = copy.deepcopy(config)
+        values = {'foo': 'bar'}
+        inputs = derived_config['inputs']
+        for i in inputs:
+            i['value'] = values.get(i['name'], i['default'])
+        inputs.append({'name': 'deploy_signal_transport',
+                       'type': 'String',
+                       'value': 'NO_SIGNAL'})
+
+        configs = {
+            '0ff2e903-78d7-4cca-829e-233af3dae705': config,
+            '48e8ade1-9196-42d5-89a2-f709fde42632': config,
+            '9966c8e7-bc9c-42de-aa7d-f2447a952cb2': derived_config,
+        }
+
+        def copy_config(context, config_id):
+            config = configs[config_id].copy()
+            config['id'] = config_id
+            return config
+
+        self.rpc_client.show_software_config.side_effect = copy_config
+
         return config
 
     def mock_software_component(self):
@@ -281,7 +327,11 @@ class SoftwareDeploymentTest(common.HeatTestCase):
             }],
             'outputs': [],
         }
-        self.rpc_client.show_software_config.return_value = config
+
+        def copy_config(*args, **kwargs):
+            return config.copy()
+
+        self.rpc_client.show_software_config.side_effect = copy_config
         return config
 
     def mock_derived_software_config(self):
@@ -290,11 +340,11 @@ class SoftwareDeploymentTest(common.HeatTestCase):
         return sc
 
     def mock_deployment(self):
-        sd = {
+        mock_sd = {
             'config_id': '9966c8e7-bc9c-42de-aa7d-f2447a952cb2'
         }
-        self.rpc_client.create_software_deployment.return_value = sd
-        return sd
+        self.rpc_client.create_software_deployment.return_value = mock_sd
+        return mock_sd
 
     def test_handle_create(self):
         self._create_stack(self.template_no_signal)
@@ -319,6 +369,12 @@ class SoftwareDeploymentTest(common.HeatTestCase):
                 'name': 'bar',
                 'type': 'String',
                 'value': 'baz'
+            }, {
+                'default': 'default_value',
+                'name': 'trigger_replace',
+                'replace_on_change': True,
+                'type': 'String',
+                'value': 'default_value'
             }, {
                 'name': 'bink',
                 'type': 'String',
@@ -416,7 +472,7 @@ class SoftwareDeploymentTest(common.HeatTestCase):
                 'value': 'bar'
             }],
             'options': None,
-            'outputs': None
+            'outputs': [],
         }, call_arg)
 
         self.assertEqual(
@@ -549,13 +605,13 @@ class SoftwareDeploymentTest(common.HeatTestCase):
 
     def test_check_create_complete(self):
         self._create_stack(self.template)
-        sd = self.mock_deployment()
-        self.rpc_client.show_software_deployment.return_value = sd
+        mock_sd = self.mock_deployment()
+        self.rpc_client.show_software_deployment.return_value = mock_sd
 
-        sd['status'] = self.deployment.COMPLETE
-        self.assertTrue(self.deployment.check_create_complete(sd))
-        sd['status'] = self.deployment.IN_PROGRESS
-        self.assertFalse(self.deployment.check_create_complete(sd))
+        mock_sd['status'] = self.deployment.COMPLETE
+        self.assertTrue(self.deployment.check_create_complete(mock_sd))
+        mock_sd['status'] = self.deployment.IN_PROGRESS
+        self.assertFalse(self.deployment.check_create_complete(mock_sd))
 
     def test_check_create_complete_none(self):
         self._create_stack(self.template)
@@ -563,14 +619,14 @@ class SoftwareDeploymentTest(common.HeatTestCase):
 
     def test_check_update_complete(self):
         self._create_stack(self.template)
-        sd = self.mock_deployment()
-        self.rpc_client.show_software_deployment.return_value = sd
+        mock_sd = self.mock_deployment()
+        self.rpc_client.show_software_deployment.return_value = mock_sd
 
-        sd['status'] = self.deployment.COMPLETE
-        self.assertTrue(self.deployment.check_update_complete(sd))
+        mock_sd['status'] = self.deployment.COMPLETE
+        self.assertTrue(self.deployment.check_update_complete(mock_sd))
 
-        sd['status'] = self.deployment.IN_PROGRESS
-        self.assertFalse(self.deployment.check_update_complete(sd))
+        mock_sd['status'] = self.deployment.IN_PROGRESS
+        self.assertFalse(self.deployment.check_update_complete(mock_sd))
 
     def test_check_update_complete_none(self):
         self._create_stack(self.template)
@@ -578,14 +634,14 @@ class SoftwareDeploymentTest(common.HeatTestCase):
 
     def test_check_suspend_complete(self):
         self._create_stack(self.template)
-        sd = self.mock_deployment()
-        self.rpc_client.show_software_deployment.return_value = sd
+        mock_sd = self.mock_deployment()
+        self.rpc_client.show_software_deployment.return_value = mock_sd
 
-        sd['status'] = self.deployment.COMPLETE
-        self.assertTrue(self.deployment.check_suspend_complete(sd))
+        mock_sd['status'] = self.deployment.COMPLETE
+        self.assertTrue(self.deployment.check_suspend_complete(mock_sd))
 
-        sd['status'] = self.deployment.IN_PROGRESS
-        self.assertFalse(self.deployment.check_suspend_complete(sd))
+        mock_sd['status'] = self.deployment.IN_PROGRESS
+        self.assertFalse(self.deployment.check_suspend_complete(mock_sd))
 
     def test_check_suspend_complete_none(self):
         self._create_stack(self.template)
@@ -593,14 +649,14 @@ class SoftwareDeploymentTest(common.HeatTestCase):
 
     def test_check_resume_complete(self):
         self._create_stack(self.template)
-        sd = self.mock_deployment()
-        self.rpc_client.show_software_deployment.return_value = sd
+        mock_sd = self.mock_deployment()
+        self.rpc_client.show_software_deployment.return_value = mock_sd
 
-        sd['status'] = self.deployment.COMPLETE
-        self.assertTrue(self.deployment.check_resume_complete(sd))
+        mock_sd['status'] = self.deployment.COMPLETE
+        self.assertTrue(self.deployment.check_resume_complete(mock_sd))
 
-        sd['status'] = self.deployment.IN_PROGRESS
-        self.assertFalse(self.deployment.check_resume_complete(sd))
+        mock_sd['status'] = self.deployment.IN_PROGRESS
+        self.assertFalse(self.deployment.check_resume_complete(mock_sd))
 
     def test_check_resume_complete_none(self):
         self._create_stack(self.template)
@@ -608,20 +664,20 @@ class SoftwareDeploymentTest(common.HeatTestCase):
 
     def test_check_create_complete_error(self):
         self._create_stack(self.template)
-        sd = {
+        mock_sd = {
             'status': self.deployment.FAILED,
             'status_reason': 'something wrong'
         }
-        self.rpc_client.show_software_deployment.return_value = sd
+        self.rpc_client.show_software_deployment.return_value = mock_sd
         err = self.assertRaises(
-            exc.Error, self.deployment.check_create_complete, sd)
+            exc.Error, self.deployment.check_create_complete, mock_sd)
         self.assertEqual(
             'Deployment to server failed: something wrong', six.text_type(err))
 
     def test_handle_delete(self):
         self._create_stack(self.template)
-        sd = self.mock_deployment()
-        self.rpc_client.show_software_deployment.return_value = sd
+        mock_sd = self.mock_deployment()
+        self.rpc_client.show_software_deployment.return_value = mock_sd
 
         self.deployment.resource_id = 'c8a19429-7fde-47ea-a42f-40045488226c'
         self.deployment.handle_delete()
@@ -633,21 +689,22 @@ class SoftwareDeploymentTest(common.HeatTestCase):
     def test_handle_delete_resource_id_is_None(self):
         self._create_stack(self.template_delete_suspend_resume)
         self.mock_software_config()
-        sd = self.mock_deployment()
-        self.assertEqual(sd, self.deployment.handle_delete())
+        mock_sd = self.mock_deployment()
+        self.assertEqual(mock_sd, self.deployment.handle_delete())
 
     def test_delete_complete(self):
         self._create_stack(self.template_delete_suspend_resume)
 
         self.mock_software_config()
         derived_sc = self.mock_derived_software_config()
-        sd = self.mock_deployment()
+        mock_sd = self.mock_deployment()
+        mock_sd['server_id'] = 'b509edfb-1448-4b57-8cb1-2e31acccbb8a'
 
         self.deployment.resource_id = 'c8a19429-7fde-47ea-a42f-40045488226c'
 
-        self.rpc_client.show_software_deployment.return_value = sd
-        self.rpc_client.update_software_deployment.return_value = sd
-        self.assertEqual(sd, self.deployment.handle_delete())
+        self.rpc_client.show_software_deployment.return_value = mock_sd
+        self.rpc_client.update_software_deployment.return_value = mock_sd
+        self.assertEqual(mock_sd, self.deployment.handle_delete())
         self.assertEqual({
             'deployment_id': 'c8a19429-7fde-47ea-a42f-40045488226c',
             'action': 'DELETE',
@@ -657,11 +714,34 @@ class SoftwareDeploymentTest(common.HeatTestCase):
             'status_reason': 'Deploy data available'},
             self.rpc_client.update_software_deployment.call_args[1])
 
-        sd['status'] = self.deployment.IN_PROGRESS
-        self.assertFalse(self.deployment.check_delete_complete(sd))
+        mock_sd['status'] = self.deployment.IN_PROGRESS
+        self.assertFalse(self.deployment.check_delete_complete(mock_sd))
 
-        sd['status'] = self.deployment.COMPLETE
-        self.assertTrue(self.deployment.check_delete_complete(sd))
+        mock_sd['status'] = self.deployment.COMPLETE
+        self.assertTrue(self.deployment.check_delete_complete(mock_sd))
+
+    def test_delete_complete_missing_server(self):
+        """Tests deleting a deployment when the server disappears"""
+        self._create_stack(self.template_delete_suspend_resume)
+
+        self.mock_software_config()
+        mock_sd = self.mock_deployment()
+        mock_sd['server_id'] = 'b509edfb-1448-4b57-8cb1-2e31acccbb8a'
+
+        # Simulate Nova not knowing about the server
+        mock_get_server = self.patchobject(
+            nova.NovaClientPlugin, 'get_server',
+            side_effect=exc.EntityNotFound)
+
+        self.deployment.resource_id = 'c8a19429-7fde-47ea-a42f-40045488226c'
+
+        self.rpc_client.show_software_deployment.return_value = mock_sd
+        self.rpc_client.update_software_deployment.return_value = mock_sd
+
+        mock_sd['status'] = self.deployment.COMPLETE
+        self.assertTrue(self.deployment.check_delete_complete(mock_sd))
+
+        mock_get_server.assert_called_once_with(mock_sd['server_id'])
 
     def test_handle_delete_notfound(self):
         self._create_stack(self.template)
@@ -670,9 +750,9 @@ class SoftwareDeploymentTest(common.HeatTestCase):
 
         self.mock_software_config()
         derived_sc = self.mock_derived_software_config()
-        sd = self.mock_deployment()
-        sd['config_id'] = derived_sc['id']
-        self.rpc_client.show_software_deployment.return_value = sd
+        mock_sd = self.mock_deployment()
+        mock_sd['config_id'] = derived_sc['id']
+        self.rpc_client.show_software_deployment.return_value = mock_sd
 
         nf = exc.NotFound
         self.rpc_client.delete_software_deployment.side_effect = nf
@@ -698,9 +778,9 @@ class SoftwareDeploymentTest(common.HeatTestCase):
         # in this case also need to call the _delete_resource(),
         # otherwise the sd data will residue in db
         self._create_stack(self.template)
-        sd = self.mock_deployment()
+        mock_sd = self.mock_deployment()
         self.deployment.resource_id = 'c8a19429-7fde-47ea-a42f-40045488226c'
-        self.rpc_client.show_software_deployment.return_value = sd
+        self.rpc_client.show_software_deployment.return_value = mock_sd
         self.assertTrue(self.deployment.check_delete_complete())
         self.assertEqual(
             (self.ctx, self.deployment.resource_id),
@@ -710,10 +790,10 @@ class SoftwareDeploymentTest(common.HeatTestCase):
         self._create_stack(self.template)
 
         self.mock_derived_software_config()
-        sd = self.mock_deployment()
+        mock_sd = self.mock_deployment()
         rsrc = self.stack['deployment_mysql']
 
-        self.rpc_client.show_software_deployment.return_value = sd
+        self.rpc_client.show_software_deployment.return_value = mock_sd
         self.deployment.resource_id = 'c8a19429-7fde-47ea-a42f-40045488226c'
         config_id = '0ff2e903-78d7-4cca-829e-233af3dae705'
         prop_diff = {
@@ -747,14 +827,96 @@ class SoftwareDeploymentTest(common.HeatTestCase):
             'status_reason': u'Deploy data available'},
             self.rpc_client.update_software_deployment.call_args[1])
 
+    def test_handle_update_no_replace_on_change(self):
+        self._create_stack(self.template)
+
+        self.mock_software_config()
+        self.mock_derived_software_config()
+        mock_sd = self.mock_deployment()
+        rsrc = self.stack['deployment_mysql']
+
+        self.rpc_client.show_software_deployment.return_value = mock_sd
+        self.deployment.resource_id = 'c8a19429-7fde-47ea-a42f-40045488226c'
+        prop_diff = {
+            'input_values': {'trigger_replace': 'default_value'},
+        }
+        props = copy.copy(rsrc.properties.data)
+        props.update(prop_diff)
+        snippet = rsrc_defn.ResourceDefinition(rsrc.name, rsrc.type(), props)
+
+        self.deployment.handle_update(snippet, None, prop_diff)
+
+        self.assertEqual({
+            'deployment_id': 'c8a19429-7fde-47ea-a42f-40045488226c',
+            'action': 'UPDATE',
+            'config_id': '9966c8e7-bc9c-42de-aa7d-f2447a952cb2',
+            'input_values': {'trigger_replace': 'default_value'},
+            'status': 'IN_PROGRESS',
+            'status_reason': u'Deploy data available'},
+            self.rpc_client.update_software_deployment.call_args[1])
+
+        self.assertEqual([
+            {
+                'default': 'baa',
+                'name': 'foo',
+                'type': 'String',
+                'value': 'baa'
+            }, {
+                'default': 'baz',
+                'name': 'bar',
+                'type': 'String',
+                'value': 'baz'
+            }, {
+                'default': 'default_value',
+                'name': 'trigger_replace',
+                'replace_on_change': True,
+                'type': 'String',
+                'value': 'default_value'
+            }],
+            self.rpc_client.create_software_config.call_args[1]['inputs'][:3])
+
+    def test_handle_update_replace_on_change(self):
+        self._create_stack(self.template)
+
+        self.mock_software_config()
+        self.mock_derived_software_config()
+        mock_sd = self.mock_deployment()
+        rsrc = self.stack['deployment_mysql']
+
+        self.rpc_client.show_software_deployment.return_value = mock_sd
+        self.deployment.resource_id = 'c8a19429-7fde-47ea-a42f-40045488226c'
+        prop_diff = {
+            'input_values': {'trigger_replace': 'new_value'},
+        }
+        props = copy.copy(rsrc.properties.data)
+        props.update(prop_diff)
+        snippet = rsrc_defn.ResourceDefinition(rsrc.name, rsrc.type(), props)
+
+        self.assertRaises(resource.UpdateReplace,
+                          self.deployment.handle_update,
+                          snippet, None, prop_diff)
+
+    def test_handle_update_with_update_only(self):
+        self._create_stack(self.template_update_only)
+        rsrc = self.stack['deployment_mysql']
+        prop_diff = {
+            'input_values': {'foo': 'different'}
+        }
+        props = copy.copy(rsrc.properties.data)
+        props.update(prop_diff)
+        snippet = rsrc_defn.ResourceDefinition(rsrc.name, rsrc.type(), props)
+        self.deployment.handle_update(
+            json_snippet=snippet, tmpl_diff=None, prop_diff=prop_diff)
+        self.rpc_client.show_software_deployment.assert_not_called()
+
     def test_handle_suspend_resume(self):
         self._create_stack(self.template_delete_suspend_resume)
 
         self.mock_software_config()
         derived_sc = self.mock_derived_software_config()
-        sd = self.mock_deployment()
+        mock_sd = self.mock_deployment()
 
-        self.rpc_client.show_software_deployment.return_value = sd
+        self.rpc_client.show_software_deployment.return_value = mock_sd
         self.deployment.resource_id = 'c8a19429-7fde-47ea-a42f-40045488226c'
 
         # first, handle the suspend
@@ -769,11 +931,11 @@ class SoftwareDeploymentTest(common.HeatTestCase):
             'status_reason': 'Deploy data available'},
             self.rpc_client.update_software_deployment.call_args[1])
 
-        sd['status'] = 'IN_PROGRESS'
-        self.assertFalse(self.deployment.check_suspend_complete(sd))
+        mock_sd['status'] = 'IN_PROGRESS'
+        self.assertFalse(self.deployment.check_suspend_complete(mock_sd))
 
-        sd['status'] = 'COMPLETE'
-        self.assertTrue(self.deployment.check_suspend_complete(sd))
+        mock_sd['status'] = 'COMPLETE'
+        self.assertTrue(self.deployment.check_suspend_complete(mock_sd))
 
         # now, handle the resume
         self.deployment.handle_resume()
@@ -787,11 +949,11 @@ class SoftwareDeploymentTest(common.HeatTestCase):
             'status_reason': 'Deploy data available'},
             self.rpc_client.update_software_deployment.call_args[1])
 
-        sd['status'] = 'IN_PROGRESS'
-        self.assertFalse(self.deployment.check_resume_complete(sd))
+        mock_sd['status'] = 'IN_PROGRESS'
+        self.assertFalse(self.deployment.check_resume_complete(mock_sd))
 
-        sd['status'] = 'COMPLETE'
-        self.assertTrue(self.deployment.check_resume_complete(sd))
+        mock_sd['status'] = 'COMPLETE'
+        self.assertTrue(self.deployment.check_resume_complete(mock_sd))
 
     def test_handle_signal_ok_zero(self):
         self._create_stack(self.template)
@@ -859,7 +1021,7 @@ class SoftwareDeploymentTest(common.HeatTestCase):
         self.assertEqual(details, ca[2])
         self.assertIsNotNone(ca[3])
 
-        # Test bug 1332355, where details contains a translateable message
+        # Test bug 1332355, where details contains a translatable message
         details = {'failed': _('need more memory.')}
         ret = self.deployment.handle_signal(details)
         self.assertEqual('deployment failed', ret)
@@ -901,7 +1063,7 @@ class SoftwareDeploymentTest(common.HeatTestCase):
 
     def test_fn_get_att(self):
         self._create_stack(self.template)
-        sd = {
+        mock_sd = {
             'outputs': [
                 {'name': 'failed', 'error_output': True},
                 {'name': 'foo'}
@@ -914,7 +1076,7 @@ class SoftwareDeploymentTest(common.HeatTestCase):
             },
             'status': self.deployment.COMPLETE
         }
-        self.rpc_client.show_software_deployment.return_value = sd
+        self.rpc_client.show_software_deployment.return_value = mock_sd
         self.assertEqual('bar', self.deployment.FnGetAtt('foo'))
         self.assertEqual('A thing happened',
                          self.deployment.FnGetAtt('deploy_stdout'))
@@ -936,11 +1098,11 @@ class SoftwareDeploymentTest(common.HeatTestCase):
     def test_fn_get_att_error(self):
         self._create_stack(self.template)
 
-        sd = {
+        mock_sd = {
             'outputs': [],
             'output_values': {'foo': 'bar'},
         }
-        self.rpc_client.show_software_deployment.return_value = sd
+        self.rpc_client.show_software_deployment.return_value = mock_sd
 
         err = self.assertRaises(
             exc.InvalidTemplateAttribute,
@@ -953,10 +1115,10 @@ class SoftwareDeploymentTest(common.HeatTestCase):
         self._create_stack(self.template)
 
         self.mock_software_config()
-        sd = self.mock_deployment()
+        mock_sd = self.mock_deployment()
         rsrc = self.stack['deployment_mysql']
 
-        self.rpc_client.show_software_deployment.return_value = sd
+        self.rpc_client.show_software_deployment.return_value = mock_sd
         self.deployment.resource_id = 'c8a19429-7fde-47ea-a42f-40045488226c'
         config_id = '0ff2e903-78d7-4cca-829e-233af3dae705'
         prop_diff = {'config': config_id}
@@ -978,10 +1140,10 @@ class SoftwareDeploymentTest(common.HeatTestCase):
         self._create_stack(self.template)
 
         self.mock_software_component()
-        sd = self.mock_deployment()
+        mock_sd = self.mock_deployment()
         rsrc = self.stack['deployment_mysql']
 
-        self.rpc_client.show_software_deployment.return_value = sd
+        self.rpc_client.show_software_deployment.return_value = mock_sd
         self.deployment.resource_id = 'c8a19429-7fde-47ea-a42f-40045488226c'
         config_id = '0ff2e903-78d7-4cca-829e-233af3dae705'
         prop_diff = {'config': config_id}
@@ -1026,9 +1188,9 @@ class SoftwareDeploymentTest(common.HeatTestCase):
             'outputs': [],
         }
         self.rpc_client.show_software_config.return_value = config
-        sd = self.mock_deployment()
+        mock_sd = self.mock_deployment()
 
-        self.rpc_client.show_software_deployment.return_value = sd
+        self.rpc_client.show_software_deployment.return_value = mock_sd
 
         self.assertIsNotNone(self.deployment.handle_create())
         self.assertIsNone(self.deployment.handle_delete())
@@ -1144,6 +1306,17 @@ class SoftwareDeploymentTest(common.HeatTestCase):
     def test_get_zaqar_queue(self):
         dep_data = {}
 
+        zc = mock.MagicMock()
+        zcc = self.patch(
+            'heat.engine.clients.os.zaqar.ZaqarClientPlugin.create_for_tenant')
+        zcc.return_value = zc
+
+        mock_queue = mock.MagicMock()
+        zc.queue.return_value = mock_queue
+
+        signed_data = {"signature": "hi", "expires": "later"}
+        mock_queue.signed_url.return_value = signed_data
+
         self._create_stack(self.template_zaqar_signal)
 
         def data_set(key, value, redact=False):
@@ -1158,6 +1331,8 @@ class SoftwareDeploymentTest(common.HeatTestCase):
 
         queue_id = self.deployment._get_zaqar_signal_queue_id()
         self.assertEqual(queue_id, dep_data['zaqar_signal_queue_id'])
+        self.assertEqual(jsonutils.dumps(signed_data),
+                         dep_data['zaqar_queue_signed_url_data'])
 
         self.assertEqual(queue_id,
                          self.deployment._get_zaqar_signal_queue_id())
@@ -1199,6 +1374,31 @@ class SoftwareDeploymentTest(common.HeatTestCase):
         self.deployment._delete_zaqar_signal_queue()
         self.assertEqual(2, len(self.deployment.data_delete.mock_calls))
 
+    def test_server_exists(self):
+        # Setup
+        self._create_stack(self.template_delete_suspend_resume)
+        mock_sd = {'server_id': 'b509edfb-1448-4b57-8cb1-2e31acccbb8a'}
+
+        # For a success case, this doesn't raise an exception
+        self.patchobject(nova.NovaClientPlugin, 'get_server')
+
+        # Test
+        result = self.deployment._server_exists(mock_sd)
+        self.assertTrue(result)
+
+    def test_server_exists_no_server(self):
+        # Setup
+        self._create_stack(self.template_delete_suspend_resume)
+        mock_sd = {'server_id': 'b509edfb-1448-4b57-8cb1-2e31acccbb8a'}
+
+        # For a success case, this doesn't raise an exception
+        self.patchobject(nova.NovaClientPlugin, 'get_server',
+                         side_effect=exc.EntityNotFound)
+
+        # Test
+        result = self.deployment._server_exists(mock_sd)
+        self.assertFalse(result)
+
 
 class SoftwareDeploymentGroupTest(common.HeatTestCase):
 
@@ -1423,3 +1623,79 @@ class SoftwareDeploymentGroupTest(common.HeatTestCase):
         snip = stack.t.resource_definitions(stack)['deploy_mysql']
         resg = sd.SoftwareDeploymentGroup('deploy_mysql', snip, stack)
         self.assertIsNone(resg.validate())
+
+
+class SDGReplaceTest(common.HeatTestCase):
+    template = {
+        'heat_template_version': '2013-05-23',
+        'resources': {
+            'deploy_mysql': {
+                'type': 'OS::Heat::SoftwareDeploymentGroup',
+                'properties': {
+                    'config': 'config_uuid',
+                    'servers': {'server1': 'uuid1', 'server2': 'uuid2'},
+                    'input_values': {'foo': 'bar'},
+                    'name': '10_config'
+                }
+            }
+        }
+    }
+
+    # 1. existing > batch_size
+    # 2. existing < batch_size
+    # 3. count > existing
+    # 4. count < exiting
+    # 5. with pause_sec
+
+    scenarios = [
+        ('1', dict(count=2,
+                   existing=['0', '1'], batch_size=1,
+                   pause_sec=0, tasks=2)),
+        ('2', dict(count=4,
+                   existing=['0', '1'], batch_size=3,
+                   pause_sec=0, tasks=2)),
+        ('3', dict(count=3,
+                   existing=['0', '1'], batch_size=2,
+                   pause_sec=0, tasks=2)),
+        ('4', dict(count=2,
+                   existing=['0', '1', '2'], batch_size=2,
+                   pause_sec=0, tasks=1)),
+        ('5', dict(count=2,
+                   existing=['0', '1'], batch_size=1,
+                   pause_sec=1, tasks=3))]
+
+    def get_fake_nested_stack(self, names):
+        nested_t = '''
+        heat_template_version: 2015-04-30
+        description: Resource Group
+        resources:
+        '''
+        resource_snip = '''
+        '%s':
+            type: SoftwareDeployment
+            properties:
+              foo: bar
+        '''
+        resources = [nested_t]
+        for res_name in names:
+            resources.extend([resource_snip % res_name])
+
+        nested_t = ''.join(resources)
+        return utils.parse_stack(template_format.parse(nested_t))
+
+    def setUp(self):
+        super(SDGReplaceTest, self).setUp()
+        self.stack = utils.parse_stack(self.template)
+        snip = self.stack.t.resource_definitions(self.stack)['deploy_mysql']
+        self.group = sd.SoftwareDeploymentGroup('deploy_mysql',
+                                                snip, self.stack)
+        self.group.update_with_template = mock.Mock()
+        self.group.check_update_complete = mock.Mock()
+
+    def test_rolling_updates(self):
+        self.group._nested = self.get_fake_nested_stack(self.existing)
+        self.group.get_size = mock.Mock(return_value=self.count)
+        tasks = self.group._replace(0, self.batch_size,
+                                    self.pause_sec)
+        self.assertEqual(self.tasks,
+                         len(tasks))
