@@ -21,10 +21,13 @@ from oslo_serialization import jsonutils
 import six
 
 from heat.common import exception
+from heat.common import identifier
 from heat.common import template_format
+from heat.engine import resource
 from heat.engine.resources import stack_resource
 from heat.engine import stack as parser
 from heat.engine import template as templatem
+from heat.objects import raw_template
 from heat.objects import stack as stack_object
 from heat.objects import stack_lock
 from heat.tests import common
@@ -32,9 +35,8 @@ from heat.tests import generic_resource as generic_rsrc
 from heat.tests import utils
 
 
-ws_res_snippet = {"HeatTemplateFormatVersion": "2012-12-12",
-                  "Type": "StackResourceType",
-                  "metadata": {
+ws_res_snippet = {"Type": "StackResourceType",
+                  "Metadata": {
                       "key": "value",
                       "some": "more stuff"}}
 
@@ -200,18 +202,10 @@ class StackResourceTest(StackResourceBaseTest):
         ret = self.parent_resource.prepare_abandon()
         self.assertEqual({}, ret)
 
-    def test_abandon_nested_not_deleted(self):
-        delete_nested = self.patchobject(self.parent_resource, 'delete_nested')
-
-        self.parent_stack.prepare_abandon()
-        self.parent_stack.delete(abandon=True)
-
-        self.assertEqual(0, delete_nested.call_count)
-
     def test_abandon_nested_sends_rpc_abandon(self):
         rpcc = mock.Mock()
         self.parent_resource.rpc_client = rpcc
-        self.parent_resource.nested = mock.MagicMock()
+        self.parent_resource.resource_id = 'fake_id'
 
         self.parent_resource.prepare_abandon()
         self.parent_resource.delete_nested()
@@ -228,7 +222,7 @@ class StackResourceTest(StackResourceBaseTest):
         """
         self.parent_stack.t.files["foo"] = "bar"
         parsed_t = self.parent_resource._parse_child_template(self.templ, None)
-        self.assertEqual({"foo": "bar"}, parsed_t.files)
+        self.assertEqual({"foo": "bar"}, parsed_t.files.files)
 
     @mock.patch('heat.engine.environment.get_child_environment')
     @mock.patch.object(stack_resource.parser, 'Stack')
@@ -320,7 +314,7 @@ class StackResourceTest(StackResourceBaseTest):
         self.parent_resource.child_params = mock.Mock(return_value={})
         self.parent_resource.preview()
         self.stack = self.parent_resource.nested()
-        self.assertEqual({"foo": "bar"}, self.stack.t.files)
+        self.assertEqual({"foo": "bar"}, self.stack.t.files.files)
 
     def test_preview_validates_nested_resources(self):
         parent_t = self.parent_stack.t
@@ -467,9 +461,13 @@ class StackResourceTest(StackResourceBaseTest):
         self.assertEqual(4, rsrc.FnGetAtt(rsrc.CURRENT_SIZE))
 
     def test__validate_nested_resources_checks_num_of_resources(self):
-        stack_resource.cfg.CONF.set_override('max_resources_per_stack', 2)
+        stack_resource.cfg.CONF.set_override('max_resources_per_stack', 2,
+                                             enforce_type=True)
         tmpl = {'HeatTemplateFormatVersion': '2012-12-12',
-                'Resources': [1]}
+                'Resources': {
+                    'r': {
+                        'Type': 'OS::Heat::None'
+                    }}}
         template = stack_resource.template.Template(tmpl)
         root_resources = mock.Mock(return_value=2)
         self.parent_resource.stack.total_resources = root_resources
@@ -509,7 +507,7 @@ class StackResourceTest(StackResourceBaseTest):
         self.assertIsNone(self.parent_resource.delete_nested())
 
     def test_delete_nested_not_found_nested_stack(self):
-        self.parent_resource._nested = mock.MagicMock()
+        self.parent_resource.resource_id = 'fake_id'
         rpcc = mock.Mock()
         self.parent_resource.rpc_client = rpcc
         rpcc.return_value.delete_stack = mock.Mock(
@@ -543,7 +541,7 @@ class StackResourceTest(StackResourceBaseTest):
         self.parent_resource.state_set(self.parent_resource.INIT,
                                        self.parent_resource.FAILED)
         self.parent_resource._nested = None
-        self.assertRaises(exception.UpdateReplace,
+        self.assertRaises(resource.UpdateReplace,
                           self.parent_resource._needs_update,
                           self.parent_resource.t,
                           self.parent_resource.t,
@@ -560,7 +558,7 @@ class StackResourceTest(StackResourceBaseTest):
         self.parent_resource.state_set(self.parent_resource.INIT,
                                        self.parent_resource.COMPLETE)
         self.parent_resource._nested = None
-        self.assertRaises(exception.UpdateReplace,
+        self.assertRaises(resource.UpdateReplace,
                           self.parent_resource._needs_update,
                           self.parent_resource.t,
                           self.parent_resource.t,
@@ -581,7 +579,7 @@ class StackResourceTest(StackResourceBaseTest):
         self.parent_resource.nested = mock.MagicMock(return_value=self.nested)
         self.parent_resource._nested = self.nested
 
-        self.assertRaises(exception.UpdateReplace,
+        self.assertRaises(resource.UpdateReplace,
                           self.parent_resource._needs_update,
                           self.parent_resource.t,
                           self.parent_resource.t,
@@ -624,69 +622,62 @@ class StackResourceLimitTest(StackResourceBaseTest):
 
 class StackResourceAttrTest(StackResourceBaseTest):
     def test_get_output_ok(self):
-        nested = self.m.CreateMockAnything()
-        self.m.StubOutWithMock(stack_resource.StackResource, 'nested')
-        stack_resource.StackResource.nested().AndReturn(nested)
-        nested.outputs = {"key": "value"}
-        nested.output('key').AndReturn("value")
-        self.m.ReplayAll()
+        self.parent_resource.nested_identifier = mock.Mock()
+        self.parent_resource.nested_identifier.return_value = {'foo': 'bar'}
+
+        self.parent_resource._rpc_client = mock.MagicMock()
+        output = {'outputs': [{'output_key': 'key', 'output_value': 'value'}]}
+        self.parent_resource._rpc_client.show_stack.return_value = [output]
 
         self.assertEqual("value", self.parent_resource.get_output("key"))
 
-        self.m.VerifyAll()
-
     def test_get_output_key_not_found(self):
-        nested = self.m.CreateMockAnything()
-        self.m.StubOutWithMock(stack_resource.StackResource, 'nested')
-        stack_resource.StackResource.nested().AndReturn(nested)
-        nested.outputs = {}
-        self.m.ReplayAll()
+        self.parent_resource.nested_identifier = mock.Mock()
+        self.parent_resource.nested_identifier.return_value = {'foo': 'bar'}
+
+        self.parent_resource._rpc_client = mock.MagicMock()
+        output = {'outputs': []}
+        self.parent_resource._rpc_client.show_stack.return_value = [output]
 
         self.assertRaises(exception.InvalidTemplateAttribute,
                           self.parent_resource.get_output,
                           "key")
 
-        self.m.VerifyAll()
-
     def test_resolve_attribute_string(self):
-        nested = self.m.CreateMockAnything()
-        self.m.StubOutWithMock(stack_resource.StackResource, 'nested')
-        stack_resource.StackResource.nested().AndReturn(nested)
-        nested.outputs = {'key': 'value'}
-        nested.output('key').AndReturn('value')
-        self.m.ReplayAll()
+        self.parent_resource.nested_identifier = mock.Mock()
+        self.parent_resource.nested_identifier.return_value = {'foo': 'bar'}
+
+        self.parent_resource._rpc_client = mock.MagicMock()
+        output = {'outputs': [{'output_key': 'key', 'output_value': 'value'}]}
+        self.parent_resource._rpc_client.show_stack.return_value = [output]
 
         self.assertEqual('value',
                          self.parent_resource._resolve_attribute("key"))
 
-        self.m.VerifyAll()
-
     def test_resolve_attribute_dict(self):
-        nested = self.m.CreateMockAnything()
-        self.m.StubOutWithMock(stack_resource.StackResource, 'nested')
-        stack_resource.StackResource.nested().AndReturn(nested)
-        nested.outputs = {'key': {'a': 1, 'b': 2}}
-        nested.output('key').AndReturn({'a': 1, 'b': 2})
-        self.m.ReplayAll()
+        self.parent_resource.nested_identifier = mock.Mock()
+        self.parent_resource.nested_identifier.return_value = {'foo': 'bar'}
+
+        self.parent_resource._rpc_client = mock.MagicMock()
+        output = {'outputs': [{'output_key': 'key',
+                               'output_value': {'a': 1, 'b': 2}}]}
+        self.parent_resource._rpc_client.show_stack.return_value = [output]
 
         self.assertEqual({'a': 1, 'b': 2},
                          self.parent_resource._resolve_attribute("key"))
 
-        self.m.VerifyAll()
-
     def test_resolve_attribute_list(self):
-        nested = self.m.CreateMockAnything()
-        self.m.StubOutWithMock(stack_resource.StackResource, 'nested')
-        stack_resource.StackResource.nested().AndReturn(nested)
-        nested.outputs = {"key": [1, 2, 3]}
-        nested.output('key').AndReturn([1, 2, 3])
-        self.m.ReplayAll()
+        self.parent_resource.nested_identifier = mock.Mock()
+        self.parent_resource.nested_identifier.return_value = {'foo': 'bar'}
+
+        self.parent_resource._rpc_client = mock.MagicMock()
+        output = {'outputs': [{'output_key': 'key',
+                               'output_value': [1, 2, 3]}]}
+        self.parent_resource._rpc_client.show_stack.return_value = [output]
 
         self.assertEqual([1, 2, 3],
                          self.parent_resource._resolve_attribute("key"))
 
-        self.m.VerifyAll()
-
     def test_validate_nested_stack(self):
         self.parent_resource.child_template = mock.Mock(return_value='foo')
         self.parent_resource.child_params = mock.Mock(return_value={})
@@ -751,6 +742,7 @@ class StackResourceCheckCompleteTest(StackResourceBaseTest):
         self.mock_status.assert_called_once_with(
             self.parent_resource.context, self.parent_resource.resource_id)
         self.mock_lock.assert_called_once_with(
+            self.parent_resource.context,
             self.parent_resource.resource_id)
 
     def test_state_err(self):
@@ -833,12 +825,24 @@ class WithTemplateTest(StackResourceBaseTest):
                        adopt_data={'template': 'foo', 'environment': 'eee'})),
     ]
 
+    class IntegerMatch(object):
+        def __eq__(self, other):
+            if getattr(self, 'match', None) is not None:
+                return other == self.match
+            if not isinstance(other, six.integer_types):
+                return False
+
+            self.match = other
+            return True
+
+        def __ne__(self, other):
+            return not self.__eq__(other)
+
     def test_create_with_template(self):
         child_env = {'parameter_defaults': {},
                      'event_sinks': [],
                      'parameters': self.params,
-                     'resource_registry': {'resources': {}},
-                     'encrypted_param_names': []}
+                     'resource_registry': {'resources': {}}}
         self.parent_resource.child_params = mock.Mock(
             return_value=self.params)
         res_name = self.parent_resource.physical_resource_name()
@@ -848,15 +852,24 @@ class WithTemplateTest(StackResourceBaseTest):
         self.parent_resource.create_with_template(
             self.empty_temp, user_params=self.params,
             timeout_mins=self.timeout_mins, adopt_data=self.adopt_data)
-        adopt_data_str = None
-        if self.adopt_data:
+        if self.adopt_data is not None:
             adopt_data_str = json.dumps(self.adopt_data)
+            tmpl_args = {
+                'template': self.empty_temp.t,
+                'params': child_env,
+                'files': {},
+            }
+        else:
+            adopt_data_str = None
+            tmpl_args = {
+                'template_id': self.IntegerMatch(),
+                'template': None,
+                'params': None,
+                'files': None,
+            }
         rpcc.return_value._create_stack.assert_called_once_with(
             self.ctx,
             stack_name=res_name,
-            template=self.empty_temp.t,
-            params=child_env,
-            files={},
             args={'disable_rollback': True,
                   'adopt_stack_data': adopt_data_str,
                   'timeout_mins': self.timeout_mins},
@@ -865,50 +878,148 @@ class WithTemplateTest(StackResourceBaseTest):
             parent_resource_name='test',
             user_creds_id='uc123',
             owner_id=self.parent_stack.id,
-            nested_depth=1)
+            nested_depth=1,
+            **tmpl_args)
 
-    def test_update_with_template(self):
-        nested = mock.MagicMock()
-        nested.updated_time = 'now_time'
-        nested.state = ('CREATE', 'COMPLETE')
-        nested.identifier.return_value = {'stack_identifier':
-                                          'stack-identifier'}
-        self.parent_resource.nested = mock.MagicMock(return_value=nested)
-        self.parent_resource._nested = nested
+    def test_create_with_template_failure(self):
+        class StackValidationFailed_Remote(exception.StackValidationFailed):
+            pass
 
         child_env = {'parameter_defaults': {},
                      'event_sinks': [],
                      'parameters': self.params,
-                     'resource_registry': {'resources': {}},
-                     'encrypted_param_names': []}
+                     'resource_registry': {'resources': {}}}
+        self.parent_resource.child_params = mock.Mock(
+            return_value=self.params)
+        res_name = self.parent_resource.physical_resource_name()
+        rpcc = mock.Mock()
+        self.parent_resource.rpc_client = rpcc
+        remote_exc = StackValidationFailed_Remote(message='oops')
+        rpcc.return_value._create_stack.side_effect = remote_exc
+        self.assertRaises(exception.ResourceFailure,
+                          self.parent_resource.create_with_template,
+                          self.empty_temp, user_params=self.params,
+                          timeout_mins=self.timeout_mins,
+                          adopt_data=self.adopt_data)
+        if self.adopt_data is not None:
+            adopt_data_str = json.dumps(self.adopt_data)
+            tmpl_args = {
+                'template': self.empty_temp.t,
+                'params': child_env,
+                'files': {},
+            }
+        else:
+            adopt_data_str = None
+            tmpl_args = {
+                'template_id': self.IntegerMatch(),
+                'template': None,
+                'params': None,
+                'files': None,
+            }
+        rpcc.return_value._create_stack.assert_called_once_with(
+            self.ctx,
+            stack_name=res_name,
+            args={'disable_rollback': True,
+                  'adopt_stack_data': adopt_data_str,
+                  'timeout_mins': self.timeout_mins},
+            environment_files=None,
+            stack_user_project_id='aprojectid',
+            parent_resource_name='test',
+            user_creds_id='uc123',
+            owner_id=self.parent_stack.id,
+            nested_depth=1,
+            **tmpl_args)
+        if self.adopt_data is None:
+            stored_tmpl_id = tmpl_args['template_id'].match
+            self.assertIsNotNone(stored_tmpl_id)
+            self.assertRaises(exception.NotFound,
+                              raw_template.RawTemplate.get_by_id,
+                              self.ctx, stored_tmpl_id)
+
+    def test_update_with_template(self):
+        if self.adopt_data is not None:
+            return
+        ident = identifier.HeatIdentifier(self.ctx.tenant_id, 'fake_name',
+                                          'pancakes')
+        self.parent_resource.resource_id = ident.stack_id
+        self.parent_resource.nested_identifier = mock.Mock(return_value=ident)
+
         self.parent_resource.child_params = mock.Mock(
             return_value=self.params)
         rpcc = mock.Mock()
         self.parent_resource.rpc_client = rpcc
-        rpcc.return_value._create_stack.return_value = {'stack_id': 'pancakes'}
-        self.parent_resource.update_with_template(
-            self.empty_temp, user_params=self.params,
-            timeout_mins=self.timeout_mins)
-        rpcc.return_value.update_stack.assert_called_once_with(
+        rpcc.return_value._update_stack.return_value = dict(ident)
+
+        status = ('CREATE', 'COMPLETE', '', 'now_time')
+        with self.patchobject(stack_object.Stack, 'get_status',
+                              return_value=status):
+            self.parent_resource.update_with_template(
+                self.empty_temp, user_params=self.params,
+                timeout_mins=self.timeout_mins)
+
+        rpcc.return_value._update_stack.assert_called_once_with(
             self.ctx,
-            stack_identity={'stack_identifier': 'stack-identifier'},
-            template=self.empty_temp.t,
-            params=child_env,
-            files={},
+            stack_identity=dict(ident),
+            template_id=self.IntegerMatch(),
+            template=None,
+            params=None,
+            files=None,
             args={'timeout_mins': self.timeout_mins})
 
+    def test_update_with_template_failure(self):
+        class StackValidationFailed_Remote(exception.StackValidationFailed):
+            pass
+
+        if self.adopt_data is not None:
+            return
+        ident = identifier.HeatIdentifier(self.ctx.tenant_id, 'fake_name',
+                                          'pancakes')
+        self.parent_resource.resource_id = ident.stack_id
+        self.parent_resource.nested_identifier = mock.Mock(return_value=ident)
+
+        self.parent_resource.child_params = mock.Mock(
+            return_value=self.params)
+        rpcc = mock.Mock()
+        self.parent_resource.rpc_client = rpcc
+        remote_exc = StackValidationFailed_Remote(message='oops')
+        rpcc.return_value._update_stack.side_effect = remote_exc
+
+        status = ('CREATE', 'COMPLETE', '', 'now_time')
+        with self.patchobject(stack_object.Stack, 'get_status',
+                              return_value=status):
+            self.assertRaises(exception.ResourceFailure,
+                              self.parent_resource.update_with_template,
+                              self.empty_temp, user_params=self.params,
+                              timeout_mins=self.timeout_mins)
+
+        template_id = self.IntegerMatch()
+        rpcc.return_value._update_stack.assert_called_once_with(
+            self.ctx,
+            stack_identity=dict(ident),
+            template_id=template_id,
+            template=None,
+            params=None,
+            files=None,
+            args={'timeout_mins': self.timeout_mins})
+        self.assertIsNotNone(template_id.match)
+        self.assertRaises(exception.NotFound,
+                          raw_template.RawTemplate.get_by_id,
+                          self.ctx, template_id.match)
+
 
 class RaiseLocalException(StackResourceBaseTest):
 
     def test_heat_exception(self):
         local = exception.StackValidationFailed(message='test')
         self.assertRaises(exception.StackValidationFailed,
-                          self.parent_resource.raise_local_exception, local)
+                          self.parent_resource.translate_remote_exceptions,
+                          local)
 
     def test_messaging_timeout(self):
         local = msg_exceptions.MessagingTimeout('took too long')
         self.assertRaises(msg_exceptions.MessagingTimeout,
-                          self.parent_resource.raise_local_exception, local)
+                          self.parent_resource.translate_remote_exceptions,
+                          local)
 
     def test_remote_heat_ex(self):
         class StackValidationFailed_Remote(exception.StackValidationFailed):
@@ -916,4 +1027,5 @@ class RaiseLocalException(StackResourceBaseTest):
 
         local = StackValidationFailed_Remote(message='test')
         self.assertRaises(exception.ResourceFailure,
-                          self.parent_resource.raise_local_exception, local)
+                          self.parent_resource.translate_remote_exceptions,
+                          local)
